from _typeshed import Incomplete
from enum import IntEnum
from typing import Any, ClassVar
from typing_extensions import Literal

from ._binary import o8 as o8
from .ImageFile import ImageFile

logger: Any
is_cid: Any
MAX_TEXT_CHUNK: Any
MAX_TEXT_MEMORY: Any

class Disposal(IntEnum):
    OP_NONE: int
    OP_BACKGROUND: int
    OP_PREVIOUS: int

APNG_DISPOSE_OP_NONE: Literal[Disposal.OP_NONE]
APNG_DISPOSE_OP_BACKGROUND: Literal[Disposal.OP_BACKGROUND]
APNG_DISPOSE_OP_PREVIOUS: Literal[Disposal.OP_PREVIOUS]

class Blend(IntEnum):
    OP_SOURCE: int
    OP_OVER: int

APNG_BLEND_OP_SOURCE: Literal[Blend.OP_SOURCE]
APNG_BLEND_OP_OVER: Literal[Blend.OP_OVER]

class ChunkStream:
    fp: Any
    queue: Any
    def __init__(self, fp) -> None: ...
    def read(self): ...
    def __enter__(self): ...
    def __exit__(self, *args) -> None: ...
    def close(self) -> None: ...
    def push(self, cid, pos, length) -> None: ...
    def call(self, cid, pos, length): ...
    def crc(self, cid, data) -> None: ...
    def crc_skip(self, cid, data) -> None: ...
    def verify(self, endchunk: bytes = ...): ...

class iTXt(str):
    lang: Any
    tkey: Any
    @staticmethod
    def __new__(cls, text, lang: Incomplete | None = ..., tkey: Incomplete | None = ...): ...

class PngInfo:
    chunks: Any
    def __init__(self) -> None: ...
    def add(self, cid, data, after_idat: bool = ...) -> None: ...
    def add_itxt(self, key, value, lang: str = ..., tkey: str = ..., zip: bool = ...) -> None: ...
    def add_text(self, key, value, zip: bool = ...): ...

class PngStream(ChunkStream):
    im_info: Any
    im_text: Any
    im_size: Any
    im_mode: Any
    im_tile: Any
    im_palette: Any
    im_custom_mimetype: Any
    im_n_frames: Any
    rewind_state: Any
    text_memory: int
    def __init__(self, fp) -> None: ...
    def check_text_memory(self, chunklen) -> None: ...
    def save_rewind(self) -> None: ...
    def rewind(self) -> None: ...
    def chunk_iCCP(self, pos, length): ...
    def chunk_IHDR(self, pos, length): ...
    im_idat: Any
    def chunk_IDAT(self, pos, length) -> None: ...
    def chunk_IEND(self, pos, length) -> None: ...
    def chunk_PLTE(self, pos, length): ...
    def chunk_tRNS(self, pos, length): ...
    def chunk_gAMA(self, pos, length): ...
    def chunk_cHRM(self, pos, length): ...
    def chunk_sRGB(self, pos, length): ...
    def chunk_pHYs(self, pos, length): ...
    def chunk_tEXt(self, pos, length): ...
    def chunk_zTXt(self, pos, length): ...
    def chunk_iTXt(self, pos, length): ...
    def chunk_eXIf(self, pos, length): ...
    def chunk_acTL(self, pos, length): ...
    def chunk_fcTL(self, pos, length): ...
    def chunk_fdAT(self, pos, length): ...

class PngImageFile(ImageFile):
    format: ClassVar[Literal["PNG"]]
    format_description: ClassVar[str]
    @property
    def text(self): ...
    fp: Any
    def verify(self) -> None: ...
    def seek(self, frame) -> None: ...
    def tell(self): ...
    decoderconfig: Any
    def load_prepare(self) -> None: ...
    def load_read(self, read_bytes): ...
    png: Any
    im: Any
    pyaccess: Any
    def load_end(self) -> None: ...
    def getexif(self): ...

def putchunk(fp, cid, *data) -> None: ...

class _idat:
    fp: Any
    chunk: Any
    def __init__(self, fp, chunk) -> None: ...
    def write(self, data) -> None: ...

class _fdat:
    fp: Any
    chunk: Any
    seq_num: Any
    def __init__(self, fp, chunk, seq_num) -> None: ...
    def write(self, data) -> None: ...

def getchunks(im, **params): ...



from typing import Any, ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

class Jpeg2KImageFile(ImageFile):
    format: ClassVar[Literal["JPEG2000"]]
    format_description: ClassVar[str]
    reduce: Any
    tile: Any
    def load(self): ...



from collections.abc import Mapping
from enum import IntEnum

TAGS: Mapping[int, str]
GPSTAGS: Mapping[int, str]

class Interop(IntEnum):
    InteropIndex: int
    InteropVersion: int
    RelatedImageFileFormat: int
    RelatedImageWidth: int
    RleatedImageHeight: int

class IFD(IntEnum):
    Exif: int
    GPSInfo: int
    Makernote: int
    Interop: int
    IFD1: int

class LightSource(IntEnum):
    Unknown: int
    Daylight: int
    Fluorescent: int
    Tungsten: int
    Flash: int
    Fine: int
    Cloudy: int
    Shade: int
    DaylightFluorescent: int
    DayWhiteFluorescent: int
    CoolWhiteFluorescent: int
    WhiteFluorescent: int
    StandardLightA: int
    StandardLightB: int
    StandardLightC: int
    D55: int
    D65: int
    D75: int
    D50: int
    ISO: int
    Other: int



from typing import Any

class ContainerIO:
    fh: Any
    pos: int
    offset: Any
    length: Any
    def __init__(self, file, offset, length) -> None: ...
    def isatty(self): ...
    def seek(self, offset, mode=...) -> None: ...
    def tell(self): ...
    def read(self, n: int = ...): ...
    def readline(self): ...
    def readlines(self): ...



from typing import Any, ClassVar
from typing_extensions import Literal, TypeAlias

from .ImageFile import ImageFile

SUPPORTED: bool
_XMP_Tags: TypeAlias = dict[str, str | _XMP_Tags]

class WebPImageFile(ImageFile):
    format: ClassVar[Literal["WEBP"]]
    format_description: ClassVar[str]
    def getxmp(self) -> _XMP_Tags: ...
    def seek(self, frame) -> None: ...
    fp: Any
    tile: Any
    def load(self): ...
    def tell(self): ...



from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

PALETTE: bytes

class XVThumbImageFile(ImageFile):
    format: ClassVar[Literal["XVThumb"]]
    format_description: ClassVar[str]



from enum import IntEnum
from typing import Any, ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile, PyDecoder

class Format(IntEnum):
    JPEG: int

BLP_FORMAT_JPEG: Literal[Format.JPEG]

class Encoding(IntEnum):
    UNCOMPRESSED: int
    DXT: int
    UNCOMPRESSED_RAW_BGRA: int

BLP_ENCODING_UNCOMPRESSED: Literal[Encoding.UNCOMPRESSED]
BLP_ENCODING_DXT: Literal[Encoding.DXT]
BLP_ENCODING_UNCOMPRESSED_RAW_BGRA: Literal[Encoding.UNCOMPRESSED_RAW_BGRA]

class AlphaEncoding(IntEnum):
    DXT1: int
    DXT3: int
    DXT5: int

BLP_ALPHA_ENCODING_DXT1: Literal[AlphaEncoding.DXT1]
BLP_ALPHA_ENCODING_DXT3: Literal[AlphaEncoding.DXT3]
BLP_ALPHA_ENCODING_DXT5: Literal[AlphaEncoding.DXT5]

def unpack_565(i): ...
def decode_dxt1(data, alpha: bool = ...): ...
def decode_dxt3(data): ...
def decode_dxt5(data): ...

class BLPFormatError(NotImplementedError): ...

class BlpImageFile(ImageFile):
    format: ClassVar[Literal["BLP"]]
    format_description: ClassVar[str]

class _BLPBaseDecoder(PyDecoder):
    magic: Any
    def decode(self, buffer): ...

class BLP1Decoder(_BLPBaseDecoder): ...
class BLP2Decoder(_BLPBaseDecoder): ...



from typing import Any

from .ContainerIO import ContainerIO

class TarIO(ContainerIO):
    fh: Any
    def __init__(self, tarfile, file) -> None: ...
    def __enter__(self): ...
    def __exit__(self, *args) -> None: ...
    def close(self) -> None: ...



from _typeshed import Incomplete
from typing import Any

class Stat:
    h: Any
    bands: Any
    def __init__(self, image_or_list, mask: Incomplete | None = ...) -> None: ...
    def __getattr__(self, id: str): ...

Global = Stat



from _typeshed import Incomplete
from typing import Any

class Iterator:
    im: Any
    position: Any
    def __init__(self, im) -> None: ...
    def __getitem__(self, ix): ...
    def __iter__(self): ...
    def __next__(self): ...

def all_frames(im, func: Incomplete | None = ...): ...



from _typeshed import Incomplete
from typing import Any

class PhotoImage:
    tk: Any
    def __init__(self, image: Incomplete | None = ..., size: Incomplete | None = ..., **kw) -> None: ...
    def __del__(self) -> None: ...
    def width(self): ...
    def height(self): ...
    def paste(self, im, box: Incomplete | None = ...) -> None: ...

class BitmapImage:
    def __init__(self, image: Incomplete | None = ..., **kw) -> None: ...
    def __del__(self) -> None: ...
    def width(self): ...
    def height(self): ...

def getimage(photo): ...



from _typeshed import Incomplete
from typing import Any, ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

enable_jpeg2k: Any
HEADERSIZE: int

def nextheader(fobj): ...
def read_32t(fobj, start_length, size): ...
def read_32(fobj, start_length, size): ...
def read_mk(fobj, start_length, size): ...
def read_png_or_jpeg2000(fobj, start_length, size): ...

class IcnsFile:
    SIZES: Any
    dct: Any
    fobj: Any
    def __init__(self, fobj) -> None: ...
    def itersizes(self): ...
    def bestsize(self): ...
    def dataforsize(self, size): ...
    def getimage(self, size: Incomplete | None = ...): ...

class IcnsImageFile(ImageFile):
    format: ClassVar[Literal["ICNS"]]
    format_description: ClassVar[str]
    @property
    def size(self): ...
    @size.setter
    def size(self, value) -> None: ...
    best_size: Any
    im: Any
    mode: Any
    def load(self) -> None: ...



from _typeshed import Incomplete
from typing import Any

from .Image import Image

class ImagePalette:
    mode: Any
    rawmode: Any
    palette: Any
    colors: Any
    dirty: Any
    def __init__(self, mode: str = ..., palette: Incomplete | None = ..., size: int = ...) -> None: ...
    def copy(self) -> ImagePalette: ...
    def getdata(self): ...
    def tobytes(self) -> bytes: ...
    tostring = tobytes
    def getcolor(self, color: tuple[int, int, int], image: Image | None = ...) -> int: ...
    def save(self, fp) -> None: ...

def raw(rawmode, data): ...
def make_linear_lut(black, white): ...
def make_gamma_lut(exp): ...
def negative(mode: str = ...): ...
def random(mode: str = ...): ...
def sepia(white: str = ...): ...
def wedge(mode: str = ...): ...
def load(filename): ...



from _typeshed import Incomplete

from .Image import Image, _Box

def grab(
    bbox: _Box | None = ..., include_layered_windows: bool = ..., all_screens: bool = ..., xdisplay: Incomplete | None = ...
) -> Image: ...
def grabclipboard() -> Image | None: ...



from _typeshed import Incomplete, SupportsRead, SupportsWrite
from collections.abc import Callable, Iterable, Iterator, MutableMapping, Sequence
from enum import IntEnum
from pathlib import Path
from typing import Any, ClassVar, Protocol, SupportsBytes
from typing_extensions import Literal, Self, TypeAlias

from PIL.PyAccess import PyAccess

from ._imaging import (
    DEFAULT_STRATEGY as DEFAULT_STRATEGY,
    FILTERED as FILTERED,
    FIXED as FIXED,
    HUFFMAN_ONLY as HUFFMAN_ONLY,
    RLE as RLE,
)
from .ImageFilter import Filter
from .ImagePalette import ImagePalette

_Mode: TypeAlias = str
_Resample: TypeAlias = Literal[0, 1, 2, 3, 4, 5]
_Size: TypeAlias = tuple[int, int]
_Box: TypeAlias = tuple[int, int, int, int]

_ConversionMatrix: TypeAlias = (
    tuple[float, float, float, float] | tuple[float, float, float, float, float, float, float, float, float, float, float, float]
)
# `str` values are only accepted if mode="RGB" for an `Image` object
# `float` values are only accepted for certain modes such as "F"
# See https://pillow.readthedocs.io/en/stable/reference/Image.html#PIL.Image.new
_Color: TypeAlias = int | tuple[int] | tuple[int, int, int] | tuple[int, int, int, int] | str | float | tuple[float]

class _Writeable(SupportsWrite[bytes], Protocol):
    def seek(self, __offset: int) -> Any: ...

NORMAL: Literal[0]  # deprecated
SEQUENCE: Literal[1]  # deprecated
CONTAINER: Literal[2]  # deprecated

class DecompressionBombWarning(RuntimeWarning): ...
class DecompressionBombError(Exception): ...

MAX_IMAGE_PIXELS: int | None

LINEAR: Literal[Resampling.BILINEAR]  # deprecated
CUBIC: Literal[Resampling.BICUBIC]  # deprecated
ANTIALIAS: Literal[Resampling.LANCZOS]  # deprecated

class Transpose(IntEnum):
    FLIP_LEFT_RIGHT: Literal[0]
    FLIP_TOP_BOTTOM: Literal[1]
    ROTATE_90: Literal[2]
    ROTATE_180: Literal[3]
    ROTATE_270: Literal[4]
    TRANSPOSE: Literal[5]
    TRANSVERSE: Literal[6]

# All Transpose items
FLIP_LEFT_RIGHT: Literal[0]
FLIP_TOP_BOTTOM: Literal[1]
ROTATE_90: Literal[2]
ROTATE_180: Literal[3]
ROTATE_270: Literal[4]
TRANSPOSE: Literal[5]
TRANSVERSE: Literal[6]

class Transform(IntEnum):
    AFFINE: Literal[0]
    EXTENT: Literal[1]
    PERSPECTIVE: Literal[2]
    QUAD: Literal[3]
    MESH: Literal[4]

# All Transform items
AFFINE: Literal[0]
EXTENT: Literal[1]
PERSPECTIVE: Literal[2]
QUAD: Literal[3]
MESH: Literal[4]

class Resampling(IntEnum):
    NEAREST: Literal[0]
    LANCZOS: Literal[1]
    BILINEAR: Literal[2]
    BICUBIC: Literal[3]
    BOX: Literal[4]
    HAMMING: Literal[5]

# All Resampling items
NEAREST: Literal[0]
LANCZOS: Literal[1]
BILINEAR: Literal[2]
BICUBIC: Literal[3]
BOX: Literal[4]
HAMMING: Literal[5]

class Dither(IntEnum):
    NONE: Literal[0]
    ORDERED: Literal[1]
    RASTERIZE: Literal[2]
    FLOYDSTEINBERG: Literal[3]

# All Dither items
NONE: Literal[0]
ORDERED: Literal[1]
RASTERIZE: Literal[2]
FLOYDSTEINBERG: Literal[3]

class Palette(IntEnum):
    WEB: Literal[0]
    ADAPTIVE: Literal[1]

# All Palette items
WEB: Literal[0]
ADAPTIVE: Literal[1]

class Quantize(IntEnum):
    MEDIANCUT: Literal[0]
    MAXCOVERAGE: Literal[1]
    FASTOCTREE: Literal[2]
    LIBIMAGEQUANT: Literal[3]

# All Quantize items
MEDIANCUT: Literal[0]
MAXCOVERAGE: Literal[1]
FASTOCTREE: Literal[2]
LIBIMAGEQUANT: Literal[3]

ID: list[str]
OPEN: dict[str, Any]
MIME: dict[str, str]
SAVE: dict[str, Any]
SAVE_ALL: dict[str, Any]
EXTENSION: dict[str, str]
DECODERS: dict[str, Any]
ENCODERS: dict[str, Any]

MODES: list[_Mode]

def getmodebase(mode: _Mode) -> Literal["L", "RGB"]: ...
def getmodetype(mode: _Mode) -> Literal["L", "I", "F"]: ...
def getmodebandnames(mode: _Mode) -> tuple[str, ...]: ...
def getmodebands(mode: _Mode) -> int: ...
def preinit() -> None: ...
def init() -> None: ...
def coerce_e(value) -> _E: ...

class _E:
    def __init__(self, scale, data) -> None: ...
    def __add__(self, other) -> _E: ...
    def __mul__(self, other) -> _E: ...

_ImageState: TypeAlias = tuple[dict[str, Any], str, tuple[int, int], Any, bytes]

class Image:
    format: ClassVar[str | None]
    format_description: ClassVar[str | None]
    im: Any
    mode: _Mode
    palette: Any
    info: dict[Any, Any]
    readonly: int
    pyaccess: PyAccess | None
    is_animated: bool  # not present on all Image objects
    n_frames: int  # not present on all Image objects
    # Only defined after a call to save().
    encoderconfig: tuple[Incomplete, ...]
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...
    @property
    def size(self) -> tuple[int, int]: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...
    def close(self) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __getstate__(self) -> _ImageState: ...
    def __setstate__(self, state: _ImageState) -> None: ...
    def tobytes(self, encoder_name: str = ..., *args) -> bytes: ...
    def tobitmap(self, name: str = ...) -> bytes: ...
    def frombytes(self, data: bytes, decoder_name: str = ..., *args) -> None: ...
    def load(self) -> None: ...
    def verify(self) -> None: ...
    def convert(
        self,
        mode: _Mode | None = ...,
        matrix: _ConversionMatrix | None = ...,
        dither: int | None = ...,
        palette: Palette | Literal[0, 1] = ...,
        colors: int = ...,
    ) -> Image: ...
    def quantize(
        self,
        colors: int = ...,
        method: Quantize | Literal[0, 1, 2, 3] | None = ...,
        kmeans: int = ...,
        palette: Image | None = ...,
        dither: int = ...,
    ) -> Image: ...
    def copy(self) -> Image: ...
    __copy__ = copy
    def crop(self, box: _Box | None = ...) -> Image: ...
    def draft(self, mode: _Mode, size: _Size) -> None: ...
    def filter(self, filter: Filter | Callable[[], Filter]) -> Image: ...
    def getbands(self) -> tuple[str, ...]: ...
    def getbbox(self) -> tuple[int, int, int, int] | None: ...
    def getcolors(self, maxcolors: int = ...) -> list[tuple[int, int]]: ...
    def getdata(self, band: int | None = ...): ...
    def getextrema(self): ...
    def getexif(self) -> Exif: ...
    def get_child_images(self) -> list[Image]: ...
    def getim(self): ...
    def getpalette(self, rawmode: str | None = ...) -> list[int] | None: ...
    def getpixel(self, xy: tuple[int, int]): ...
    def getprojection(self) -> tuple[list[int], list[int]]: ...
    def histogram(self, mask: Image | None = ..., extrema: tuple[int, int] | tuple[float, float] | None = ...) -> list[int]: ...
    def entropy(self, mask: Image | None = ..., extrema: tuple[int, int] | tuple[float, float] | None = ...) -> float: ...
    def paste(self, im: Image | _Color, box: tuple[int, int] | _Box | None = ..., mask: Image | None = ...) -> None: ...
    def alpha_composite(self, im: Image, dest: tuple[int, int] = ..., source: tuple[int, int] = ...) -> None: ...
    def point(self, lut, mode: _Mode | None = ...) -> Image: ...
    def putalpha(self, alpha: Image | int) -> None: ...
    def putdata(self, data: Sequence[int], scale: float = ..., offset: float = ...) -> None: ...
    def putpalette(self, data: ImagePalette | bytes | Iterable[int] | SupportsBytes, rawmode: _Mode | None = ...) -> None: ...
    def putpixel(self, xy: tuple[int, int], value: _Color | list[float]) -> None: ...
    def remap_palette(self, dest_map: Iterable[int], source_palette: Sequence[int] | None = ...) -> Image: ...
    def resize(
        self,
        size: tuple[int, int],
        resample: Resampling | _Resample | None = ...,
        box: tuple[float, float, float, float] | None = ...,
        reducing_gap: float | None = ...,
    ) -> Image: ...
    def reduce(self, factor: int | tuple[int, int] | list[int], box: _Box | None = ...) -> Image: ...
    def rotate(
        self,
        angle: float,
        resample: Resampling | _Resample = ...,
        expand: bool = ...,
        center: tuple[float, float] | None = ...,
        translate: tuple[float, float] | None = ...,
        fillcolor: _Color | None = ...,
    ) -> Image: ...
    def save(
        self,
        fp: str | bytes | Path | _Writeable,
        format: str | None = ...,
        *,
        save_all: bool = ...,
        bitmap_format: Literal["bmp", "png"] = ...,  # for ICO files
        optimize: bool = ...,
        **params: Any,
    ) -> None: ...
    def seek(self, frame: int) -> None: ...
    def show(self, title: str | None = ...) -> None: ...
    def split(self) -> tuple[Image, ...]: ...
    def getchannel(self, channel: int | str) -> Image: ...
    def tell(self) -> int: ...
    def thumbnail(self, size: tuple[int, int], resample: Resampling | _Resample = ..., reducing_gap: float = ...) -> None: ...
    def transform(
        self,
        size: _Size,
        method: Transform | Literal[0, 1, 2, 3, 4],
        data=...,
        resample: Resampling | _Resample = ...,
        fill: int = ...,
        fillcolor: _Color | int | None = ...,
    ) -> Image: ...
    def transpose(self, method: Transpose | Literal[0, 1, 2, 3, 4, 5, 6]) -> Image: ...
    def effect_spread(self, distance: int) -> Image: ...
    def toqimage(self): ...
    def toqpixmap(self): ...

class ImagePointHandler: ...
class ImageTransformHandler: ...

def new(mode: _Mode, size: tuple[int, int], color: _Color = ...) -> Image: ...
def frombytes(mode: _Mode, size: tuple[int, int], data, decoder_name: str = ..., *args) -> Image: ...
def frombuffer(mode: _Mode, size: tuple[int, int], data, decoder_name: str = ..., *args) -> Image: ...
def fromarray(obj, mode: _Mode | None = ...) -> Image: ...
def fromqimage(im) -> Image: ...
def fromqpixmap(im) -> Image: ...
def open(
    fp: str | bytes | Path | SupportsRead[bytes], mode: Literal["r"] = ..., formats: list[str] | tuple[str, ...] | None = ...
) -> Image: ...
def alpha_composite(im1: Image, im2: Image) -> Image: ...
def blend(im1: Image, im2: Image, alpha: float) -> Image: ...
def composite(image1: Image, image2: Image, mask: Image) -> Image: ...
def eval(image: Image, *args) -> Image: ...
def merge(mode: _Mode, bands: Sequence[Image]) -> Image: ...
def register_open(id: str, factory, accept=...) -> None: ...
def register_mime(id: str, mimetype: str) -> None: ...
def register_save(id: str, driver) -> None: ...
def register_save_all(id: str, driver) -> None: ...
def register_extension(id: str, extension: str) -> None: ...
def register_extensions(id: str, extensions: Iterable[str]) -> None: ...
def registered_extensions() -> dict[str, str]: ...
def register_decoder(name: str, decoder) -> None: ...
def register_encoder(name: str, encoder) -> None: ...
def effect_mandelbrot(size: tuple[int, int], extent: tuple[float, float, float, float], quality: int) -> Image: ...
def effect_noise(size: tuple[int, int], sigma: float) -> Image: ...
def linear_gradient(mode: _Mode) -> Image: ...
def radial_gradient(mode: _Mode) -> Image: ...

class Exif(MutableMapping[int, Any]):
    def load(self, data: bytes) -> None: ...
    def tobytes(self, offset: int = ...) -> bytes: ...
    def get_ifd(self, tag: int): ...
    def hide_offsets(self) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, tag: int) -> Any: ...
    def __contains__(self, tag: object) -> bool: ...
    def __setitem__(self, tag: int, value: Any) -> None: ...
    def __delitem__(self, tag: int) -> None: ...
    def __iter__(self) -> Iterator[int]: ...



from _typeshed import Incomplete
from typing import Any

class Pen:
    color: Any
    width: Any
    def __init__(self, color, width: int = ..., opacity: int = ...) -> None: ...

class Brush:
    color: Any
    def __init__(self, color, opacity: int = ...) -> None: ...

class Font:
    color: Any
    font: Any
    def __init__(self, color, file, size: int = ...) -> None: ...

class Draw:
    draw: Any
    image: Any
    transform: Any
    def __init__(self, image, size: Incomplete | None = ..., color: Incomplete | None = ...) -> None: ...
    def flush(self): ...
    def render(self, op, xy, pen, brush: Incomplete | None = ...) -> None: ...
    def settransform(self, offset) -> None: ...
    def arc(self, xy, start, end, *options) -> None: ...
    def chord(self, xy, start, end, *options) -> None: ...
    def ellipse(self, xy, *options) -> None: ...
    def line(self, xy, *options) -> None: ...
    def pieslice(self, xy, start, end, *options) -> None: ...
    def polygon(self, xy, *options) -> None: ...
    def rectangle(self, xy, *options) -> None: ...
    def text(self, xy, text, font) -> None: ...
    def textsize(self, text, font): ...



from _typeshed import Incomplete
from collections.abc import MutableMapping
from numbers import Rational
from typing import Any, ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

logger: Any
READ_LIBTIFF: bool
WRITE_LIBTIFF: bool
IFD_LEGACY_API: bool
II: bytes
MM: bytes
IMAGEWIDTH: int
IMAGELENGTH: int
BITSPERSAMPLE: int
COMPRESSION: int
PHOTOMETRIC_INTERPRETATION: int
FILLORDER: int
IMAGEDESCRIPTION: int
STRIPOFFSETS: int
SAMPLESPERPIXEL: int
ROWSPERSTRIP: int
STRIPBYTECOUNTS: int
X_RESOLUTION: int
Y_RESOLUTION: int
PLANAR_CONFIGURATION: int
RESOLUTION_UNIT: int
TRANSFERFUNCTION: int
SOFTWARE: int
DATE_TIME: int
ARTIST: int
PREDICTOR: int
COLORMAP: int
TILEOFFSETS: int
SUBIFD: int
EXTRASAMPLES: int
SAMPLEFORMAT: int
JPEGTABLES: int
REFERENCEBLACKWHITE: int
COPYRIGHT: int
IPTC_NAA_CHUNK: int
PHOTOSHOP_CHUNK: int
ICCPROFILE: int
EXIFIFD: int
XMP: int
JPEGQUALITY: int
IMAGEJ_META_DATA_BYTE_COUNTS: int
IMAGEJ_META_DATA: int
COMPRESSION_INFO: Any
COMPRESSION_INFO_REV: Any
OPEN_INFO: Any
PREFIXES: Any

class IFDRational(Rational):
    def __init__(self, value, denominator: int = ...) -> None: ...
    @property
    def numerator(a): ...
    @property
    def denominator(a): ...
    def limit_rational(self, max_denominator): ...
    def __hash__(self) -> int: ...
    def __eq__(self, other): ...
    __add__: Any
    __radd__: Any
    __sub__: Any
    __rsub__: Any
    __mul__: Any
    __rmul__: Any
    __truediv__: Any
    __rtruediv__: Any
    __floordiv__: Any
    __rfloordiv__: Any
    __mod__: Any
    __rmod__: Any
    __pow__: Any
    __rpow__: Any
    __pos__: Any
    __neg__: Any
    __abs__: Any
    __trunc__: Any
    __lt__: Any
    __gt__: Any
    __le__: Any
    __ge__: Any
    __bool__: Any
    __ceil__: Any
    __floor__: Any
    __round__: Any

class ImageFileDirectory_v2(MutableMapping[int, Any]):
    group: int | None
    tagtype: dict[int, int]
    def __init__(self, ifh: bytes = ..., prefix: bytes | None = ..., group: int | None = ...) -> None: ...
    @property
    def prefix(self) -> bytes: ...
    @property
    def offset(self) -> int | None: ...
    @property
    def legacy_api(self) -> bool: ...
    def reset(self) -> None: ...
    def named(self): ...
    def __len__(self) -> int: ...
    def __getitem__(self, tag): ...
    def __contains__(self, tag): ...
    def __setitem__(self, tag, value) -> None: ...
    def __delitem__(self, tag) -> None: ...
    def __iter__(self): ...
    def load_byte(self, data, legacy_api: bool = ...): ...
    def write_byte(self, data): ...
    def load_string(self, data, legacy_api: bool = ...): ...
    def write_string(self, value: int | str | bytes) -> bytes: ...
    def load_rational(self, data, legacy_api: bool = ...): ...
    def write_rational(self, *values): ...
    def load_undefined(self, data, legacy_api: bool = ...): ...
    def write_undefined(self, value): ...
    def load_signed_rational(self, data, legacy_api: bool = ...): ...
    def write_signed_rational(self, *values): ...
    def load(self, fp) -> None: ...
    def tobytes(self, offset: int = ...): ...
    def save(self, fp): ...

class ImageFileDirectory_v1(ImageFileDirectory_v2):
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def tags(self): ...
    @property
    def tagdata(self): ...
    tagtype: dict[int, int]
    @classmethod
    def from_v2(cls, original): ...
    def to_v2(self): ...
    def __contains__(self, tag): ...
    def __len__(self) -> int: ...
    def __iter__(self): ...
    def __setitem__(self, tag, value) -> None: ...
    def __getitem__(self, tag): ...

ImageFileDirectory = ImageFileDirectory_v1

class TiffImageFile(ImageFile):
    format: ClassVar[Literal["TIFF", "MIC"]]
    format_description: ClassVar[str]
    tag_v2: Any
    tag: Any
    def __init__(self, fp: Incomplete | None = ..., filename: Incomplete | None = ...) -> None: ...
    @property
    def n_frames(self): ...
    im: Any
    def seek(self, frame) -> None: ...
    def tell(self): ...
    def load(self): ...
    def load_end(self) -> None: ...

SAVE_INFO: Any

class AppendingTiffWriter:
    fieldSizes: Any
    Tags: Any
    f: Any
    close_fp: bool
    name: Any
    beginning: Any
    def __init__(self, fn, new: bool = ...) -> None: ...
    whereToWriteNewIFDOffset: Any
    offsetOfNewPage: int
    IIMM: Any
    isFirst: bool
    def setup(self) -> None: ...
    def finalize(self) -> None: ...
    def newFrame(self) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_value, traceback): ...
    def tell(self): ...
    def seek(self, offset, whence=...): ...
    def goToEnd(self) -> None: ...
    endian: Any
    longFmt: Any
    shortFmt: Any
    tagFormat: Any
    def setEndian(self, endian) -> None: ...
    def skipIFDs(self) -> None: ...
    def write(self, data): ...
    def readShort(self): ...
    def readLong(self): ...
    def rewriteLastShortToLong(self, value) -> None: ...
    def rewriteLastShort(self, value) -> None: ...
    def rewriteLastLong(self, value) -> None: ...
    def writeShort(self, value) -> None: ...
    def writeLong(self, value) -> None: ...
    def close(self) -> None: ...
    def fixIFD(self) -> None: ...
    def fixOffsets(self, count, isShort: bool = ..., isLong: bool = ...) -> None: ...



from typing import Any

EPSILON: float

def linear(middle, pos): ...
def curved(middle, pos): ...
def sine(middle, pos): ...
def sphere_increasing(middle, pos): ...
def sphere_decreasing(middle, pos): ...

SEGMENTS: Any

class GradientFile:
    gradient: Any
    def getpalette(self, entries: int = ...): ...

class GimpGradientFile(GradientFile):
    gradient: Any
    def __init__(self, fp) -> None: ...



from _typeshed import StrOrBytesPath

from .Image import Image

LUT_SIZE: int
ROTATION_MATRIX: list[int]
MIRROR_MATRIX: list[int]

class LutBuilder:
    patterns: list[str]
    lut: bytearray
    def __init__(self, patterns: list[str] | None = ..., op_name: str | None = ...) -> None: ...
    def add_patterns(self, patterns: list[str]) -> None: ...
    def build_default_lut(self) -> None: ...
    def get_lut(self) -> bytearray: ...
    def build_lut(self) -> bytearray: ...

class MorphOp:
    lut: bytearray
    def __init__(self, lut: bytearray | None = ..., op_name: str | None = ..., patterns: list[str] | None = ...) -> None: ...
    def apply(self, image: Image) -> tuple[int, Image]: ...
    def match(self, image: Image) -> list[tuple[int, int]]: ...
    def get_on_pixels(self, image: Image) -> list[tuple[int, int]]: ...
    def load_lut(self, filename: StrOrBytesPath) -> None: ...
    def save_lut(self, filename: StrOrBytesPath) -> None: ...
    def set_lut(self, lut: bytearray) -> None: ...
